@@ -52,7 +52,7 @@
 */
 
 #ifndef gmic_version
-#define gmic_version 275
+#define gmic_version 282
 
 #ifndef gmic_pixel_type
 #define gmic_pixel_type float
@@ -78,7 +78,7 @@ namespace cimg_library {
     unsigned int _height;      // Number of image lines (dimension along the Y-axis)
     unsigned int _depth;       // Number of image slices (dimension along the Z-axis)
     unsigned int _spectrum;    // Number of image channels (dimension along the C-axis)
-    bool _is_shared;           // Tells if the data buffer is shared by another structure
+    bool _is_shared;           // Tells if the data buffer has been allocated by another object
     T *_data;                  // Pointer to the first pixel value
 
     // Destructor.
@@ -91,6 +91,14 @@ namespace cimg_library {
     gmic_image<T>& assign(const unsigned int size_x, const unsigned int size_y=1,
                           const unsigned int size_z=1, const unsigned int size_c=1);
 
+    // Create image by copying existing buffer of t values.
+    template<typename t>
+    gmic_image<T>& assign(const t *const values, const unsigned int size_x, const unsigned int size_y=1,
+                          const unsigned int size_z=1, const unsigned int size_c=1);
+
+    gmic_image<T>& assign(const T *const values, const unsigned int size_x, const unsigned int size_y,
+                          const unsigned int size_z, const unsigned int size_c, const bool is_shared);
+
     // Pixel access.
     operator T*() {
       return _data;
@@ -176,9 +184,14 @@ inline bool *gmic_abort_ptr(bool *const p_is_abort);
 #endif // #ifdef cimg_use_abort
 
 template<typename T>
-inline double gmic_mp_ext(char *const str, void *const p_list, const T& pixel_type);
-#define cimg_mp_ext_function(str) return ::gmic_mp_ext(str._data,&mp.listout,(T)0)
+inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type);
+#define cimg_mp_call_function(str) return ::gmic_mp_call(str,&mp.listout,(T)0)
 
+template<typename Ts, typename T>
+inline double gmic_mp_store(const Ts *const ptr,
+                            const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
+                            const char *const str, void *const p_list, const T& pixel_type);
+#define cimg_mp_store_function(ptr,w,h,d,s,str) return ::gmic_mp_store(ptr,w,h,d,s,str,&mp.listout,(T)0)
 #ifndef cimg_display
 #define cimg_display 0
 #endif // #ifndef cimg_display
@@ -216,8 +229,7 @@ struct gmic {
   ~gmic();
 
   // Constructors.
-  template<typename T=gmic_pixel_type>
-  gmic(const T& pixel_type=(T)0);
+  gmic();
 
   template<typename T=gmic_pixel_type>
   gmic(const char *const commands_line,
@@ -251,7 +263,12 @@ struct gmic {
   template<typename T>
   static bool search_sorted(const char *const str, const T& list, const unsigned int length, unsigned int &out_ind);
   template<typename T>
-  static double mp_ext(char *const str, void *const p_list, const T& pixel_type);
+  static double mp_call(char *const str, void *const p_list, const T& pixel_type);
+  template<typename Ts, typename T>
+  static double mp_store(const Ts *const ptr,
+                         const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
+                         const char *const str, void *const p_list, const T& pixel_type);
+  static bool get_debug_info(const char *const s, unsigned int &line_number, unsigned int &file_number);
   static int _levenshtein(const char *const s, const char *const t,
                           gmic_image<int>& d, const int i, const int j);
   static int levenshtein(const char *const s, const char *const t);
@@ -278,9 +295,11 @@ struct gmic {
                            const unsigned int *const variables_sizes=0);
 
   gmic& add_commands(const char *const data_commands, const char *const commands_file=0,
-                     unsigned int *count_new=0, unsigned int *count_replaced=0);
+                     unsigned int *count_new=0, unsigned int *count_replaced=0,
+                     bool *const is_entrypoint=0);
   gmic& add_commands(std::FILE *const file, const char *const filename=0,
-                     unsigned int *count_new=0, unsigned int *count_replaced=0);
+                     unsigned int *count_new=0, unsigned int *count_replaced=0,
+                     bool *const is_entrypoint=0);
 
   gmic_image<char> callstack2string(const bool _is_debug=false) const;
   gmic_image<char> callstack2string(const gmic_image<unsigned int>& callstack_selection,
@@ -405,7 +424,7 @@ struct gmic {
   unsigned long reference_time;
   unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages, debug_filename, debug_line, cimg_exception_mode;
   int verbosity,render3d, renderd3d;
-  bool is_change, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info,
+  bool allow_entrypoint, is_change, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info,
     _is_abort, *is_abort, is_abort_thread;
   const char *starting_commands_line;
 };
@@ -413,15 +432,15 @@ struct gmic {
 // Class 'gmic_exception'.
 //------------------------
 struct gmic_exception {
-  gmic_image<char> _command_help, _message;
+  gmic_image<char> _command, _message;
 
   // Constructors.
   gmic_exception() {}
 
   gmic_exception(const char *const command, const char *const message) {
     if (command) {
-      _command_help.assign((unsigned int)std::strlen(command) + 1,1,1,1);
-      std::strcpy(_command_help._data,command);
+      _command.assign((unsigned int)std::strlen(command) + 1,1,1,1);
+      std::strcpy(_command._data,command);
     }
     if (message) {
       _message.assign((unsigned int)std::strlen(message) + 1,1,1,1);
@@ -434,14 +453,20 @@ struct gmic_exception {
     return _message._data?_message._data:"";
   }
 
-  const char *command_help() const {
-    return _command_help._data?_command_help._data:"";
+  const char *command() const {
+    return _command._data?_command._data:"";
   }
 };
 
 template<typename T>
-inline double gmic_mp_ext(char *const str, void *const p_list, const T& pixel_type) {
-  return gmic::mp_ext(str,p_list,pixel_type);
+inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type) {
+  return gmic::mp_call(str,p_list,pixel_type);
+}
+template<typename Ts, typename T>
+inline double gmic_mp_store(const Ts *const ptr,
+                            const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
+                            const char *const str, void *const p_list, const T& pixel_type) {
+  return gmic::mp_store(ptr,w,h,d,s,str,p_list,pixel_type);
 }
 inline bool *gmic_abort_ptr(bool *const p_is_abort) { return gmic::abort_ptr(p_is_abort); }
 
