@@ -12,7 +12,7 @@
  *
  * This software is a computer program whose purpose is to demonstrate
  * the possibilities of the GMIC image processing language by offering the
- * choice of several manipulations on a video stream aquired from a webcam. In
+ * choice of several manipulations on a video stream acquired from a webcam. In
  * other words, ZArt is a GUI for G'MIC real-time manipulations on the output
  * of a webcam.
  *
@@ -185,8 +185,6 @@ void FilterThread::run()
         _arguments.unlock();
         c += call;
 
-        // SHOW(c);
-
         _gmic->run(c.toLatin1().constData(), _gmic_images, _gmic_images_names);
         lastCommandDuration = timeMeasure.elapsed();
 
@@ -259,7 +257,7 @@ void FilterThread::run()
         }
 
       } catch (gmic_exception & e) {
-        CImg<unsigned char> src(reinterpret_cast<unsigned char *>(_imageSource.image()->imageData), 3, _imageSource.width(), _imageSource.height(), 1, true);
+        CImg<unsigned char> src(reinterpret_cast<unsigned char *>(_imageSource.image()->ptr()), 3, _imageSource.width(), _imageSource.height(), 1, true);
         _gmic_images = src.get_permute_axes("yzcx");
         QString errorCommand = QString("-gimp_error_preview \"%1\"").arg(e.what());
 
@@ -270,7 +268,7 @@ void FilterThread::run()
           _gmic_images = src.get_permute_axes("yzcx").channel(0).resize(-100, -100, 1, 3).draw_text(10, 10, "Syntax Error", color1, color2, 0.5, 57);
         }
         std::cerr << e.what() << std::endl;
-        QSize size(_imageSource.image()->width, _imageSource.image()->height);
+        QSize size(_imageSource.image()->cols, _imageSource.image()->rows);
         if (_outputImageA->size() != size) {
           _imageMutexA->lock();
           *_outputImageA = QImage(size, QImage::Format_RGB888);
